#!/usr/bin/env python3

# Copyright 2013, 2014, 2015, 2016, 2017 Andrzej Cichocki

# This file is part of pyven.
#
# pyven is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyven is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyven.  If not, see <http://www.gnu.org/licenses/>.

import os, pyven, tests
from pyvenimpl import projectinfo, miniconda as mc

def main():
    info = projectinfo.ProjectInfo(os.getcwd())
    minicondas = [mc.pyversiontominiconda[v] for v in info['pyversions']]
    testspath = tests.__file__
    if testspath.endswith('.pyc'):
        testspath = testspath[:-1]
    for miniconda in minicondas:
        # Equivalent to running tests.py directly but with one fewer process launch:
        pyven.Launcher(info, miniconda.pyversion).check_call([testspath])

if '__main__' == __name__:
    main()



#!/usr/bin/env python3

# Copyright 2013, 2014, 2015, 2016, 2017 Andrzej Cichocki

# This file is part of pyven.
#
# pyven is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyven is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyven.  If not, see <http://www.gnu.org/licenses/>.

import os, subprocess, pyven, tests
from pyvenimpl import projectinfo, miniconda as mc

class Workspace:

    def __init__(self):
        self.workspace = os.path.dirname(os.getcwd())

    def info(self, projectname):
        return projectinfo.ProjectInfo(os.path.join(self.workspace, projectname))

    def gettransitivedeps(self, info, deps):
        deps.update(info['deps'])
        for project in info['projects']:
            self.gettransitivedeps(self.info(project), deps)

    def checkoutifnecessary(self, info, project):
        if '/' in project:
            return # Assume it's a subdirectory of the context project.
        path = os.path.join(self.workspace, project)
        if not os.path.exists(path): # Allow for diamond dependencies.
            cloneargs = ['-b', info['branch'].get(project, 'master')]
            subprocess.check_call(['git', 'clone'] + cloneargs + ["https://github.com/combatopera/%s.git" % project], cwd = self.workspace)
        info2 = projectinfo.ProjectInfo(path)
        for project2 in info2['projects']:
            self.checkoutifnecessary(info2, project2)

def main():
    workspace = Workspace()
    info = projectinfo.ProjectInfo(os.getcwd())
    for project in info['projects']:
        workspace.checkoutifnecessary(info, project)
    minicondas = [mc.pyversiontominiconda[v] for v in info['pyversions']]
    deps = set()
    workspace.gettransitivedeps(info, deps)
    workspace.gettransitivedeps(workspace.info('pyven'), deps)
    for miniconda in minicondas:
        miniconda.installifnecessary(deps)
    testspath = tests.__file__
    if testspath.endswith('.pyc'):
        testspath = testspath[:-1]
    for miniconda in minicondas:
        # Equivalent to running tests.py directly but with one fewer process launch:
        pyven.Launcher(info, miniconda.pyversion).check_call([testspath])

if '__main__' == __name__:
    main()



# Copyright 2013, 2014, 2015, 2016, 2017 Andrzej Cichocki

# This file is part of pyven.
#
# pyven is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyven is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyven.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import with_statement
import re, os, hashlib

template="""# Copyright %(years)s %(author)s

# This file is part of %(name)s.
#
# %(name)s is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# %(name)s is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with %(name)s.  If not, see <http://www.gnu.org/licenses/>.

""" # Check it ends with 2 newlines.

def mainimpl(info, paths):
    master = template % {
        'years': ', '.join(str(y) for y in info['years']),
        'author': info['author'],
        'name': info['name'],
    }
    for path in paths:
        with open(path) as f:
            text = f.read()
        if text.startswith('#!'):
            for _ in range(2):
                text = text[text.index('\n') + 1:]
        if path.endswith('.s'):
            text = re.sub('^;', '#', text, flags = re.MULTILINE)
        elif path.endswith('.h') or path.endswith('.cpp') or path.endswith('.cxx'):
            text = re.sub('^//', '#', text, flags = re.MULTILINE)
        elif path.endswith('.arid'):
            text = re.sub('^:', '#', text, flags = re.MULTILINE)
        text = text[:len(master)]
        if master != text:
            raise Exception(path)
    gplpath = os.path.join(info.projectdir, 'COPYING')
    md5 = hashlib.md5()
    with open(gplpath) as f:
        md5.update(f.read().encode('utf_8'))
    if 'd32239bcb673463ab874e80d47fae504' != md5.hexdigest():
        raise Exception(gplpath)



# Copyright 2013, 2014, 2015, 2016, 2017 Andrzej Cichocki

# This file is part of pyven.
#
# pyven is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyven is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyven.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import with_statement
import re, ast, sys

def mainimpl(paths):
    if sys.version_info[0] >= 3:
        sys.stderr.write('SKIP ')
        return
    for path in paths:
        with open(path) as f:
            text = f.read()
        for node in ast.walk(ast.parse(text)):
            if 'Div' == type(node).__name__:
                hasdiv = True
                break
        else:
            hasdiv = False
        if hasdiv == (re.search('^from __future__ import division(?: # .+)?$', text, flags = re.MULTILINE) is None):
            raise Exception(path)



#!/usr/bin/env python3

# Copyright 2013, 2014, 2015, 2016, 2017 Andrzej Cichocki

# This file is part of pyven.
#
# pyven is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyven is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyven.  If not, see <http://www.gnu.org/licenses/>.

from pyvenimpl import projectinfo, miniconda
from warnings import warn
import os, sys, subprocess, itertools, aridity

class WrongBranchException(Exception): pass

def branchornone(projectpath):
    if os.path.exists(os.path.join(projectpath, '.git')):
        process = subprocess.Popen(['git', 'rev-parse', '--abbrev-ref', 'HEAD'], cwd = projectpath, stdout = subprocess.PIPE)
        lines = process.communicate()[0].decode().splitlines()
        if process.wait():
            raise Exception
        branch, = lines
        return branch
    infopath = os.path.join(projectpath, '.pyven.arid')
    if os.path.exists(infopath):
        context = aridity.Context()
        with aridity.Repl(context) as repl:
            repl.printf(". %s", infopath)
        return context.resolved('branch').value

class Launcher:

    @classmethod
    def projectpaths(cls, workspace, info, seenpaths):
        for project in info['projects']:
            path = os.path.join(workspace, project.replace('/', os.sep))
            if path in seenpaths:
                continue
            expectedbranch = info['branch'].get(project, 'master')
            actualbranch = branchornone(os.path.join(workspace, project.split('/', 1)[0]))
            if actualbranch is None:
                warn("Unknown branch: %s" % path)
            elif actualbranch != expectedbranch:
                raise WrongBranchException("For %s expected %s but branch is: %s" % (project, expectedbranch, actualbranch))
            seenpaths.add(path)
            yield path
            if '/' in project:
                continue
            info2 = projectinfo.ProjectInfo(path)
            for path2 in cls.projectpaths(workspace, info2, seenpaths):
                yield path2

    @staticmethod
    def getenv(projectpaths):
        key = 'PYTHONPATH'
        env = os.environ.copy()
        try:
            currentpaths = [env[key]] # No need to actually split.
        except KeyError:
            currentpaths = []
        env[key] = os.pathsep.join(itertools.chain(projectpaths, currentpaths))
        return env

    def __init__(self, info, pyversion):
        workspace = os.path.dirname(info.projectdir)
        self.env = self.getenv(self.projectpaths(workspace, info, set()))
        self.pathtopython = miniconda.pyversiontominiconda[pyversion].executable()

    def replace(self, args):
        os.execvpe(self.pathtopython, [self.pathtopython] + args, self.env)

    def check_call(self, args):
        subprocess.check_call([self.pathtopython] + args, env = self.env)

def main():
    try:
        info = projectinfo.ProjectInfo(os.path.dirname(os.path.realpath(sys.argv[1])))
    except projectinfo.ProjectInfoNotFoundException:
        os.execvp(sys.executable, [sys.executable] + sys.argv[1:])
    Launcher(info, info['pyversions'][0]).replace(sys.argv[1:])

if '__main__' == __name__:
    main()



# Copyright 2013, 2014, 2015, 2016, 2017 Andrzej Cichocki

# This file is part of pyven.
#
# pyven is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyven is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyven.  If not, see <http://www.gnu.org/licenses/>.

import os, subprocess

class PathExistsException(Exception): pass

class Miniconda:

    condaversion = '4.3.21'
    opt = os.path.join(os.path.expanduser('~'), 'opt')

    def __init__(self, pyversion, title, dirname, envkey):
        self.pyversion = pyversion
        self.scriptname = "%s-%s-Linux-x86_64.sh" % (title, self.condaversion)
        self.target = os.path.join(self.opt, dirname)
        self.envkey = envkey

    def installifnecessary(self, deps):
        if self.envkey in os.environ:
            return # Already installed.
        for path in self.scriptname, self.target:
            if os.path.exists(path):
                raise PathExistsException(path) # Panic.
        subprocess.check_call(['wget', '--no-verbose', "http://repo.continuum.io/miniconda/%s" % self.scriptname])
        command = ['bash', self.scriptname, '-b', '-p', self.target]
        subprocess.check_call(command)
        os.remove(self.scriptname)
        command = [os.path.join(self.target, 'bin', 'pip'), 'install', 'pyflakes', 'nose'] # XXX: Why not pyparsing?
        command.extend(deps)
        subprocess.check_call(command)
        os.environ[self.envkey] = self.target

    def executable(self):
        return os.path.join(os.environ[self.envkey], 'bin', 'python')

class Jython:

    pyversion = 'jython'

    def executable(self):
        return 'jython'

pyversiontominiconda = {info.pyversion: info for info in [
    Miniconda(2, 'Miniconda2', 'miniconda', 'MINICONDA_HOME'),
    Miniconda(3, 'Miniconda3', 'miniconda3', 'MINICONDA3_HOME'),
    Jython(),
]}



#!/usr/bin/env pyven

# Copyright 2013, 2014, 2015, 2016, 2017 Andrzej Cichocki

# This file is part of pyven.
#
# pyven is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyven is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyven.  If not, see <http://www.gnu.org/licenses/>.

from pyvenimpl import inpkg

def main():
    inpkg.main()

if '__main__' == __name__:
    main()



#!/usr/bin/env python3

# Copyright 2013, 2014, 2015, 2016, 2017 Andrzej Cichocki

# This file is part of pyven.
#
# pyven is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyven is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyven.  If not, see <http://www.gnu.org/licenses/>.

import os, subprocess
from tests import Files

def isproject():
    for name in '.hg', '.svn', '.git':
        if os.path.exists(name):
            return True

def main():
    while not isproject():
        os.chdir('..')
    agcommand = ['ag', '--noheading', '--nobreak']
    paths = list(Files.findfiles('.py', '.pyx', '.h', '.cpp', '.ui', '.java', '.kt', '.c', '.s', '.sh'))
    for tag in 'XXX', 'TODO', 'FIXME':
        subprocess.call(agcommand + [tag + ' LATER'] + paths)
        subprocess.call(agcommand + [tag + '(?! LATER)'] + paths)

if '__main__' == __name__:
    main()



# Copyright 2013, 2014, 2015, 2016, 2017 Andrzej Cichocki

# This file is part of pyven.
#
# pyven is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyven is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyven.  If not, see <http://www.gnu.org/licenses/>.

import os

def endswithifmain(istest, lines):
    if ('    unittest.main()' if istest else '    main()') != lines[-1]:
        return False
    for i in range(len(lines) - 2, -1, -1):
        if '''if '__main__' == __name__:''' == lines[i]:
            return True
        if not lines[i].startswith('    '):
            return False
    return False

def mainimpl(paths): # TODO: Can probably be simplified now that tests are non-executable.
    for path in paths:
        executable = os.stat(path).st_mode & 0x49
        if 0 == executable:
            executable = False
        elif 0x49 == executable:
            executable = True
        else:
            raise Exception(path) # Should be all or nothing.
        basename = os.path.basename(path)
        istest = basename.startswith('test_')
        if basename not in ('tests.py', 'Test.py') and basename.lower().startswith('test') and not istest:
            raise Exception(path) # Catch bad naming.
        if istest and executable:
            raise Exception(path) # All tests should be non-executable.
        f = open(path)
        try:
            lines = f.read().splitlines()
        finally:
            f.close()
        hashbang = bool(lines) and lines[0] in (
            '#!/usr/bin/env python',
            '#!/usr/bin/env python3',
            '#!/usr/bin/env pyven',
        )
        main = bool(lines) and endswithifmain(istest, lines)
        if hashbang and main and executable: return
        # An otherwise non-executable file may have a main if it's always passed to an interpreter:
        if (not hashbang) and (not executable): return
        raise Exception(path)



# Copyright 2013, 2014, 2015, 2016, 2017 Andrzej Cichocki

# This file is part of pyven.
#
# pyven is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyven is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyven.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import with_statement
import subprocess, sys, os, re
from pyvenimpl import licheck as licheckimpl, nlcheck as nlcheckimpl, divcheck as divcheckimpl, execcheck as execcheckimpl, projectinfo
from pyvenimpl.util import stderr

def stripeol(line):
    line, = line.splitlines()
    return line

class Files:

    @staticmethod
    def findfiles(*suffixes):
        walkpath = '.'
        prefixlen = len(walkpath + os.sep)
        for dirpath, dirnames, filenames in os.walk(walkpath):
            for name in sorted(filenames):
                for suffix in suffixes:
                    if name.endswith(suffix):
                        yield os.path.join(dirpath, name)[prefixlen:]
                        break # Next name.
            dirnames.sort()

    @classmethod
    def filterfiles(cls, *suffixes):
        if os.path.exists('.hg'):
            badstatuses = set('IR ')
            for line in subprocess.Popen(['hg', 'st', '-A'] + list(cls.findfiles(*suffixes)), stdout = subprocess.PIPE).stdout:
                line = stripeol(line).decode()
                if line[0] not in badstatuses:
                    yield line[2:]
        else:
            for x in cls.findfiles(*suffixes): yield x

    def __init__(self):
        self.allsrcpaths = list(p for p in self.filterfiles('.py', '.py3', '.pyx', '.s', '.sh', '.h', '.cpp', '.cxx', '.arid') if 'setup.py' != os.path.basename(p))
        self.pypaths = [p for p in self.allsrcpaths if p.endswith('.py')]

def licheck(info, files):
    def g():
        for path in files.allsrcpaths:
            parentname = os.path.basename(os.path.dirname(path))
            if parentname != 'contrib' and not parentname.endswith('_turbo'):
                yield path
    licheckimpl.mainimpl(info, list(g()))

def nlcheck(info, files):
    nlcheckimpl.mainimpl(files.allsrcpaths)

def divcheck(info, files):
    divcheckimpl.mainimpl(files.pypaths)

def execcheck(info, files):
    execcheckimpl.mainimpl(files.pypaths)

def pyflakes(info, files):
    with open('.flakesignore') as f:
        ignores = [re.compile(stripeol(l)) for l in f]
    def accept(path):
        for pattern in ignores:
            if pattern.search(path) is not None:
                return False
        return True
    paths = [p for p in files.pypaths if accept(p)]
    if paths:
        subprocess.check_call([pathto('pyflakes')] + paths)

def pathto(executable):
    return os.path.join(os.path.dirname(os.path.realpath(sys.executable)), executable)

def main():
    while not (os.path.exists('.hg') or os.path.exists('.svn') or os.path.exists('.git')):
        os.chdir('..')
    info = projectinfo.ProjectInfo(os.getcwd())
    files = Files()
    for check in (() if info['proprietary'] else (licheck,)) + (nlcheck, divcheck, execcheck, pyflakes):
        sys.stderr.write("%s: " % check.__name__)
        check(info, files)
        stderr('OK')
    sys.exit(subprocess.call([pathto('nosetests'), '--exe', '-v', '-m', '^test_']))

if '__main__' == __name__:
    main()



import setuptools

setuptools.setup(
        name = 'pyven',
        version = '5',
        description = 'Management of PYTHONPATH for simultaneous dev of multiple projects',
        url = 'https://github.com/combatopera/pyven',
        author = 'Andrzej Cichocki',
        packages = setuptools.find_packages(),
        py_modules = ['gclean', 'tasks', 'release', 'runtests', 'travis_ci', 'pyven', 'toplevel', 'tests'],
        install_requires = ['aridity'],
        package_data = {'': ['*.pxd', '*.pyx', '*.pyxbld', '*.arid', '*.aridt']},
        scripts = ['gclean.py', 'tasks.py', 'pyven', 'release.py', 'runtests.py', 'travis_ci.py', 'pyven.py', 'toplevel.py', 'tests', 'foreignsyms'])



# Copyright 2013, 2014, 2015, 2016, 2017 Andrzej Cichocki

# This file is part of pyven.
#
# pyven is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyven is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyven.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import with_statement
import os, sys
d = os.path.dirname(os.path.realpath(__file__)) # pyvenimpl
d = os.path.dirname(d) # pyven
d = os.path.dirname(d) # workspace
sys.path.append(os.path.join(d, 'aridity'))
del d
import aridity, stat

class ProjectInfoNotFoundException(Exception): pass

def textcontent(node):
    def iterparts(node):
        value = node.nodeValue
        if value is None:
            for child in node.childNodes:
                for text in iterparts(child):
                    yield text
        else:
            yield value
    return ''.join(iterparts(node))

class ProjectInfo:

    def __init__(self, realdir):
        self.projectdir = realdir
        while True:
            infopath = os.path.join(self.projectdir, 'project.arid')
            if os.path.exists(infopath):
                break
            parent = os.path.dirname(self.projectdir)
            if parent == self.projectdir:
                raise ProjectInfoNotFoundException(realdir)
            self.projectdir = parent
        self.info = aridity.Context()
        with aridity.Repl(self.info) as repl:
            repl.printf('projects := $list()')
            repl.printf('branch := $fork()')
            repl.printf('deps := $list()')
            repl.printf('pyversions := $list()')
            repl.printf('proprietary = false')
            repl.printf('executable = false')
            repl.printf(". %s", infopath)

    def __getitem__(self, key):
        return self.info.resolved(key).unravel()

    def nextversion(self):
        import urllib.request, urllib.error, re, xml.dom.minidom as dom
        pattern = re.compile('-([0-9]+)[-.]')
        try:
            with urllib.request.urlopen("https://pypi.org/simple/%s/" % self['name']) as f:
                doc = dom.parseString(f.read())
            last = max(int(pattern.search(textcontent(a)).group(1)) for a in doc.getElementsByTagName('a'))
        except urllib.error.HTTPError as e:
            if 404 != e.code:
                raise
            last = 0
        return str(last + 1)

    def descriptionandurl(self):
        import urllib.request, json, re, subprocess
        urlpath = re.search('^git@github[.]com:(.+/.+)[.]git$', subprocess.check_output(['git', 'remote', 'get-url', 'origin'], cwd = self.projectdir).decode()).group(1)
        with urllib.request.urlopen("https://api.github.com/repos/%s" % urlpath) as f:
            return json.loads(f.read().decode())['description'], "https://github.com/%s" % urlpath

    def py_modules(self):
        suffix = '.py'
        return [name[:-len(suffix)] for name in os.listdir(self.projectdir) if name.endswith(suffix) and 'setup.py' != name and not name.startswith('test_')]

    def scripts(self):
        if not self['executable']:
            return []
        xmask = stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH
        def isscript(path):
            return os.stat(path).st_mode & xmask and not os.path.isdir(path)
        return [name for name in os.listdir(self.projectdir) if isscript(os.path.join(self.projectdir, name))]



#!/usr/bin/env python3

# Copyright 2013, 2014, 2015, 2016, 2017 Andrzej Cichocki

# This file is part of pyven.
#
# pyven is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyven is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyven.  If not, see <http://www.gnu.org/licenses/>.

from pyvenimpl.projectinfo import ProjectInfo
import os, sys, subprocess, shutil, argparse, logging

log = logging.getLogger(__name__)
setupformat = """import setuptools

setuptools.setup(
        name = %r,
        version = %r,
        description = %r,
        url = %r,
        author = %r,
        packages = setuptools.find_packages(),
        py_modules = %r,
        install_requires = %r,
        package_data = {'': ['*.pxd', '*.pyx', '*.pyxbld', '*.arid', '*.aridt']},
        scripts = %r)
"""
cfgformat = """[bdist_wheel]
universal=%s
"""

def main():
    logging.basicConfig(format = "[%(levelname)s] %(message)s", level = logging.DEBUG)
    parser = argparse.ArgumentParser()
    parser.add_argument('--upload', action = 'store_true')
    parser.add_argument('path', nargs = '?', type = os.path.abspath, default = os.getcwd())
    config = parser.parse_args()
    info = ProjectInfo(config.path)
    with open(os.path.join(info.projectdir, 'setup.py'), 'w') as f:
        f.write(setupformat % ((info['name'], info.nextversion()) + info.descriptionandurl() + (info['author'], info.py_modules(), info['deps'] + info['projects'], info.scripts())))
    with open(os.path.join(info.projectdir, 'setup.cfg'), 'w') as f:
        f.write(cfgformat % int({2, 3} <= set(info['pyversions'])))
    dist = os.path.join(info.projectdir, 'dist')
    if os.path.isdir(dist):
        shutil.rmtree(dist) # Remove any previous versions.
    subprocess.check_call([sys.executable, 'setup.py', 'sdist', 'bdist_wheel'], cwd = info.projectdir)
    if config.upload:
        subprocess.check_call([sys.executable, '-m', 'twine', 'upload'] + [os.path.join(dist, name) for name in os.listdir(dist)])
    else:
        log.warning('Upload skipped, use --upload to upload.')

if '__main__' == __name__:
    main()
